/*
 * Project: MoleCuilder
 * Description: creates and alters molecular systems
 * Copyright (C)  2010-2012 University of Bonn. All rights reserved.
 * 
 *
 *   This file is part of MoleCuilder.
 *
 *    MoleCuilder is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    MoleCuilder is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with MoleCuilder.  If not, see <http://www.gnu.org/licenses/>.
 */

/*
 * PrincipalAxisSystemAction.cpp
 *
 *  Created on: May 12, 2010
 *      Author: heber
 */

// include config.h
#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

//#include "CodePatterns/MemDebug.hpp"

#include "CodePatterns/Log.hpp"
#include "CodePatterns/Verbose.hpp"
#include "LinearAlgebra/RealSpaceMatrix.hpp"
#include "LinearAlgebra/Vector.hpp"
#include "Element/element.hpp"
#include "molecule.hpp"

#include <iostream>
#include <string>

#include "Actions/AnalysisAction/PrincipalAxisSystemAction.hpp"

using namespace MoleCuilder;

// and construct the stuff
#include "PrincipalAxisSystemAction.def"
#include "Action_impl_pre.hpp"

/** =========== define the function ====================== */
ActionState::ptr AnalysisPrincipalAxisSystemAction::performCall() {
  LOG(0, "Evaluating prinicipal axis.");
  for (World::MoleculeSelectionIterator iter = World::getInstance().beginMoleculeSelection(); iter != World::getInstance().endMoleculeSelection(); ++iter) {
    molecule *mol = iter->second;

    // get inertia tensor
    RealSpaceMatrix InertiaTensor = mol->getInertiaTensor();

		// diagonalize matrix
    Vector EigenValues = InertiaTensor.transformToEigenbasis();

    // print principal axis system
    LOG(1, "INFO: The Principal Axis System of molecule "
        << mol->getName() <<  " is:"
        << InertiaTensor << endl
        << " with eigenvalues "
        << EigenValues);
  }
  return Action::success;
}

ActionState::ptr AnalysisPrincipalAxisSystemAction::performUndo(ActionState::ptr _state) {
  return Action::success;
}

ActionState::ptr AnalysisPrincipalAxisSystemAction::performRedo(ActionState::ptr _state){
  return Action::success;
}

bool AnalysisPrincipalAxisSystemAction::canUndo() {
  return true;
}

bool AnalysisPrincipalAxisSystemAction::shouldUndo() {
  return true;
}
/** =========== end of function ====================== */
